﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace SlowHttpServer
{
    class Program
    {
        static void Main(string[] args)
        {
            using (Socket serverSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp))
            {
                serverSocket.Bind(new IPEndPoint(IPAddress.Any, 8035));
                serverSocket.Listen(5);

                while (true)
                {
                    Socket socket = serverSocket.Accept();
                    ThreadPool.QueueUserWorkItem(processChildSocket, socket);
                }
            }
        }

        static int _count = 0;

        private static void processChildSocket(object state)
        {
            int idx = Interlocked.Increment(ref _count);

            using (Socket child = state as Socket)
            {
                byte[] buf = new byte[8192];
                int len = child.Receive(buf);

                string txt = Encoding.UTF8.GetString(buf, 0, len);
                Console.WriteLine("[" + idx + "]" + DateTime.Now + Environment.NewLine + txt);

                int delay = GetDelayTime(txt);

                byte[] responseBuf = GetResponseData(idx, delay);

                child.Send(responseBuf);
                child.Close();
            }
        }

        private static int GetDelayTime(string txt)
        {
            StringReader sr = new StringReader(txt);
            int sleep = 0;
            while (true)
            {
                string line = sr.ReadLine();

                if (line == null)
                {
                    break;
                }

                int pos = line.IndexOf("Delay");
                if (pos == -1)
                {
                    continue;
                }

                int colonPos = line.IndexOf(":");
                if (colonPos == -1)
                {
                    continue;
                }

                sleep = Int32.Parse(line.Substring(colonPos + 1));
            }

            return sleep;
        }

        private static byte[] GetResponseData(int idx, int delay)
        {
            DateTime now = DateTime.Now;
            Thread.Sleep(delay);
            DateTime after = DateTime.Now;

            string header = "HTTP/1.1 200 OK\r\nContent-Type: text/html; charset=UTF-8\r\n\r\n";
            string body = "<html><body>" + idx + "<mark>" + now + "<br />\r\n" + after + "<br />\r\n HTML</mark> 웹 페이지입니다.</body></html>";
            byte[] respBuf = Encoding.UTF8.GetBytes(header + body);
            return respBuf;
        }
    }
}
